/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.buddyreplication;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.CacheSPI;
import org.jboss.cache.util.TestingUtil;
import static org.testng.AssertJUnit.assertEquals;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

/**
 * Tests basic group membership semantics
 *
 * @author <a href="mailto:manik AT jboss DOT org">Manik Surtani (manik AT jboss DOT org)</a>
 */
@Test(groups = "functional", testName = "buddyreplication.BuddyPoolBroadcastTest")
public class BuddyPoolBroadcastTest extends BuddyReplicationTestsBase
{
   private Log log = LogFactory.getLog(BuddyPoolBroadcastTest.class);


   @AfterMethod(alwaysRun = true)
   public void tearDown() throws Exception
   {
      long st = System.currentTimeMillis();
      super.tearDown();
   }

   public void test2CachesWithPoolNames() throws Exception
   {
      log.error("Running test2CachesWithPoolNames");
      List<CacheSPI<Object, Object>> caches = createCaches(2, true);
      cachesTL.set(caches);

      BuddyManager m = caches.get(0).getBuddyManager();
      Map groupMap = m.buddyPool;

      assertEquals("A", groupMap.get(caches.get(0).getLocalAddress()));
      assertEquals("B", groupMap.get(caches.get(1).getLocalAddress()));
   }

   public void testConcurrency() throws Exception
   {
         int numCaches = 4;
         List<CacheSPI<Object, Object>> caches = new ArrayList<CacheSPI<Object, Object>>(4);
         cachesTL.set(caches);
         CountDownLatch latch = new CountDownLatch(1);
         CacheStarter[] starters = new CacheStarter[numCaches];

         for (int i = 0; i < numCaches; i++)
         {
            caches.add(createCache(1, new String(new char[]{(char) ('A' + i)}), false, false));
            starters[i] = new CacheStarter("CacheStarter-" + i, latch, caches.get(i));
            starters[i].start();
         }

         // now have the lot start simultaneously
         latch.countDown();

         // allow a generous sleep time
         TestingUtil.blockUntilViewsReceived(caches.toArray(new CacheSPI[0]), 240000);


         long timeout = System.currentTimeMillis() + 5000 * numCaches;
         while (System.currentTimeMillis() < timeout)
         {
            boolean allPoolNameBroadcasted = true;
            for (int i = 0; i < numCaches; i++)
            {
               boolean receivedFromAll = caches.get(i).getBuddyManager().buddyPool.size() == numCaches;
               allPoolNameBroadcasted &= receivedFromAll;
            }
            if (allPoolNameBroadcasted)
            {
               break;
            }
            if (System.currentTimeMillis() >= timeout)
            {
               log.error("********* pool membership was not brodcasted in " + (timeout - System.currentTimeMillis()) + " millis!");

               assert false : "pool membership was not brodcasted in " + timeout + " millis!";
            }
            Thread.sleep(50);
         }

         //and now look at the state of things.
         Map map = caches.get(0).getBuddyManager().buddyPool;
         for (int i = 0; i < numCaches; i++)
         {
            if (caches.get(i) != null)
               assertEquals("Failed on cache " + i + "(" + caches.get(i).getLocalAddress() + ")", new String(new char[]{(char) ('A' + i)}), map.get(caches.get(i).getLocalAddress()));
         }

         checkConsistentPoolState(caches);
   }

   public class CacheStarter extends Thread
   {
      private CountDownLatch latch;
      private CacheSPI cache;

      public CacheStarter(String name, CountDownLatch latch, CacheSPI cache)
      {
         super(name);
         this.latch = latch;
         this.cache = cache;
      }

      public void run()
      {
         try
         {
            latch.await();
            cache.start();
         }
         catch (Exception e)
         {
            e.printStackTrace();
         }
      }
   }

}
