/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.passivation;

import java.util.Properties;

import org.jboss.cache.Fqn;
import static org.jboss.cache.factories.UnitTestCacheConfigurationFactory.*;
import org.jboss.cache.config.CacheLoaderConfig;
import org.jboss.cache.util.TestingUtil;
import org.jboss.cache.util.TestDbPropertiesFactory;
import org.testng.annotations.Test;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.AfterTest;

/**
 * Tests passivation using JDBC Cache Loader.
 * This test has MySQL hard-coded. To run it, run MySQL first: mysqld -u=root
 *
 * @author <a href="mailto:{hmesha@novell.com}">{Hany Mesha}</a>
 * @version $Id: PassivationToJDBCCacheLoaderTest.java 7306 2008-12-12 08:50:23Z mircea.markus $          
 */
@Test(groups = "functional", testName = "passivation.PassivationToJDBCCacheLoaderTest")
public class PassivationToJDBCCacheLoaderTest extends PassivationTestsBase
{

   private Properties props;
   private long durration;

   @BeforeTest
   public void createDatabase()
   {
      durration = System.currentTimeMillis();
      props = TestDbPropertiesFactory.getTestDbProperties();
   }

   @AfterTest
   public void shutDownDatabase()
   {
      TestDbPropertiesFactory.shutdownInMemoryDatabase(props);
   }

   private Properties getJDBCProps()
   {
       return props;
   }

   @AfterMethod(alwaysRun = true)
   public void tearDown() throws Exception
   {
      log.info("**** TEARING DOWN ****");
      if (loader != null) loader.remove(Fqn.ROOT);
      TestingUtil.killCaches(cache);
      TestDbPropertiesFactory.shutdownInMemoryDatabase(props);
   }


   protected void configureCache() throws Exception
   {
      CacheLoaderConfig loaderConfig = buildSingleCacheLoaderConfig(true, null, "org.jboss.cache.loader.JDBCCacheLoader",
            getJDBCProps(), false, false, false, false, false);
      cache.getConfiguration().setCacheLoaderConfig(loaderConfig);
   }
}
